"""  aidgaf-server - A simple server to handle requests from the aidgaf client. 
    This server will communicate with OpenAI to generate responses to the client."""
import json
import random

from http.server import BaseHTTPRequestHandler, HTTPServer
import idgaf
import settings
import security
from const import UTF8

default_request_body = b'{"message":{"command":"aidgaf","data":{"username":"AdamOutler"}}}'


class IDGAFServer(BaseHTTPRequestHandler):
    """ This class handles the requests from the client. """
    
    def do_GET(self):
        """ This function handles GET requests. """
        self.send_response(418)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        self.wfile.write(bytes("", UTF8))

    def do_PATCH(self):
        """ This function handles PATCH requests. """
        body = self.get_body().decode(UTF8)
        if not perform_sanity_checks(body):
            self.send_response(403)
            return
        command = json.loads(body)
        self.do_request_handling(command)

    def do_request_handling(self, command):
        """ This function handles the request. 
            Parameters:
                command: The command object received from the client. """
        print(command)
        if command['message']['command'] == 'aidgaf':
            [responseCode, json_response] = idgaf.parse_idgaf_request(command)
            json_response['hash'] = security.get_message_hash(json.dumps(command))
            response_body = json.dumps(json_response)
            self.handle_response(responseCode, response_body)
        
    def get_body(self)->bytes:
        """ This function returns the body of the request.
            Returns:
                The body of the request.
            """
        header_length = self.headers.get('Content-Length')
        request_body = default_request_body
        if header_length != None and self.headers.get('Content-Length') != None:
            request_body = self.rfile.read(
                int(self.headers.get('Content-Length')))
        str_request = str(request_body, UTF8)
        print(request_body)
        return request_body
        
        
    def handle_response(self, code, body):
        """ This function handles the response to the client.
            Parameters:
                code: The HTTP response code.
                body: The body of the response. """
        self.send_response(code)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        print("sending:"+body)
        self.wfile.write(bytes(body, UTF8))
        
def perform_sanity_checks(str_request)->bool:
    """ Performs a hash check on the message, and verifies the timestamp is valid.
        If either check fails, return False.  Otherwise, return True.
        Parameters:
            str_request: The request body as a string.
        Returns:
            True if the message is valid, False otherwise. """        
    if settings.HASHKEY is not None:
        hash = security.get_message_hash(str_request)
        if hash not in str_request:
            print("Error: hash not match")
            return False
        if not security.verify_message_time(str_request):
            print("Error: timestamp expired")
            return False
    return True




def main():
    """ This function starts the server. """
    webServer = HTTPServer(
        (settings.HOSTNAME, settings.SERVERPORT), IDGAFServer)
    print("Server started http://%s:%s" %
          (settings.HOSTNAME, settings.SERVERPORT))

    try:
        webServer.serve_forever()
    except KeyboardInterrupt:
        pass

    webServer.server_close()
    print("Server stopped.")


if __name__ == "__main__":
    """ This function starts the main method. """
    main()
